@@ -6,8 +6,7 @@ tmuxp.workspacebuilder
 
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals, with_statement)
+from __future__ import absolute_import, unicode_literals
 
 import logging
 
@@ -25,61 +24,66 @@ logger = logging.getLogger(__name__)
 
 class WorkspaceBuilder(object):
 
-    """Load workspace from session :py:obj:`dict`.
+    """
+    Load workspace from session :py:obj:`dict`.
 
     Build tmux workspace from a configuration. Creates and names windows, sets
     options, splits windows into panes.
 
     The normal phase of loading is:
 
-        1.  :term:`kaptan` imports json/yaml/ini. ``.get()`` returns python
-            :class:`dict`::
+    1. :term:`kaptan` imports json/yaml/ini. ``.get()`` returns python
+       :class:`dict`::
 
-                import kaptan
-                sconf = kaptan.Kaptan(handler='yaml')
-                sconf = sconfig.import_config(self.yaml_config).get()
+           import kaptan
+           sconf = kaptan.Kaptan(handler='yaml')
+           sconf = sconfig.import_config(self.yaml_config).get()
 
-            or from config file with extension::
+       or from config file with extension::
 
-                import kaptan
-                sconf = kaptan.Kaptan()
-                sconf = sconfig.import_config('path/to/config.yaml').get()
+           import kaptan
+           sconf = kaptan.Kaptan()
+           sconf = sconfig.import_config('path/to/config.yaml').get()
 
-            kaptan automatically detects the handler from filenames.
+       kaptan automatically detects the handler from filenames.
 
-        2.  :meth:`config.expand` sconf inline shorthand::
+    2. :meth:`config.expand` sconf inline shorthand::
 
-                from tmuxp import config
-                sconf = config.expand(sconf)
+           from tmuxp import config
+           sconf = config.expand(sconf)
 
-        3.  :meth:`config.trickle` passes down default values from session
-            -> window -> pane if applicable::
+    3. :meth:`config.trickle` passes down default values from session
+       -> window -> pane if applicable::
 
-                sconf = config.trickle(sconf)
+           sconf = config.trickle(sconf)
 
-        4.  (You are here) We will create a :class:`Session` (a real
-            ``tmux(1)`` session) and iterate through the list of windows, and
-            their panes, returning full :class:`Window` and :class:`Pane`
-            objects each step of the way::
+    4. (You are here) We will create a :class:`libtmux.Session` (a real
+       ``tmux(1)`` session) and iterate through the list of windows, and
+       their panes, returning full :class:`libtmux.Window` and
+       :class:`libtmux.Pane` objects each step of the way::
 
-                workspace = WorkspaceBuilder(sconf=sconf)
+           workspace = WorkspaceBuilder(sconf=sconf)
 
     It handles the magic of cases where the user may want to start
     a session inside tmux (when `$TMUX` is in the env variables).
-
     """
 
     def __init__(self, sconf, server=None):
-        """Initialize workspace loading.
+        """
+        Initialize workspace loading.
 
-        :todo: initialize :class:`Session` from here, in ``self.session``.
+        Parameters
+        ----------
+        sconf : dict
+            session config, includes a :py:obj:`list` of ``windows``.
 
-        :param sconf: session config, includes a :py:obj:`list` of ``windows``.
-        :type sconf: :py:obj:`dict`
-
-        :param server:
-        :type server: :class:`Server`
+        server : :class:`libtmux.Server`
+            tmux server to build session in
 
+        Notes
+        -----
+        TODO: Initialize :class:`libtmux.Session` from here, in
+        ``self.session``.
         """
 
         if not sconf:
@@ -99,50 +103,46 @@ class WorkspaceBuilder(object):
         if not exists:
             return exists
 
-        self.session = self.server.find_where(
-            {
-                'session_name': session_name
-            }
-        )
+        self.session = self.server.find_where({'session_name': session_name})
         return True
 
     def build(self, session=None):
-        """Build tmux workspace in session.
+        """
+        Build tmux workspace in session.
 
         Optionally accepts ``session`` to build with only session object.
 
-        Without ``session``, it will use :class:`Server` at ``self.server``
-        passed in on initialization to create a new Session object.
-
-        :param session: - session to build workspace in
-        :type session: :class:`Session`
+        Without ``session``, it will use :class:`libmtux.Server` at
+        ``self.server`` passed in on initialization to create a new Session
+        object.
 
+        Parameters
+        ----------
+        session : :class:`libtmux.Session`
+            session to build workspace in
         """
 
         if not session:
             if not self.server:
                 raise exc.TmuxpException(
-                    'WorkspaceBuilder.build requires server to be passed ' +
-                    'on initialization, or pass in session object to here.'
+                    'WorkspaceBuilder.build requires server to be passed '
+                    + 'on initialization, or pass in session object to here.'
                 )
 
             if self.server.has_session(self.sconf['session_name']):
                 self.session = self.server.find_where(
-                    {
-                        'session_name': self.sconf['session_name']
-                    }
+                    {'session_name': self.sconf['session_name']}
                 )
                 raise TmuxSessionExists(
-                    'Session name %s is already running.' %
-                    self.sconf['session_name']
+                    'Session name %s is already running.' % self.sconf['session_name']
                 )
             else:
                 session = self.server.new_session(
                     session_name=self.sconf['session_name']
                 )
 
-            assert(self.sconf['session_name'] == session.name)
-            assert(len(self.sconf['session_name']) > 0)
+            assert self.sconf['session_name'] == session.name
+            assert len(self.sconf['session_name']) > 0
 
         self.session = session
         self.server = session.server
@@ -151,7 +151,7 @@ class WorkspaceBuilder(object):
         assert self.server.has_session(session.name)
         assert session.id
 
-        assert(isinstance(session, Session))
+        assert isinstance(session, Session)
 
         focus = None
 
@@ -163,10 +163,7 @@ class WorkspaceBuilder(object):
                 # session start directory, if it exists.
                 if 'start_directory' in self.sconf:
                     cwd = self.sconf['start_directory']
-                run_before_script(
-                    self.sconf['before_script'],
-                    cwd=cwd
-                )
+                run_before_script(self.sconf['before_script'], cwd=cwd)
             except Exception as e:
                 self.session.kill_session()
                 raise e
@@ -181,11 +178,11 @@ class WorkspaceBuilder(object):
                 self.session.set_environment(option, value)
 
         for w, wconf in self.iter_create_windows(session):
-            assert(isinstance(w, Window))
+            assert isinstance(w, Window)
 
             focus_pane = None
             for p, pconf in self.iter_create_panes(w, wconf):
-                assert(isinstance(p, Pane))
+                assert isinstance(p, Pane)
                 p = p
 
                 if 'layout' in wconf:
@@ -206,16 +203,24 @@ class WorkspaceBuilder(object):
             focus.select_window()
 
     def iter_create_windows(self, s):
-        """Return :class:`Window` iterating through session config dict.
+        """
+        Return :class:`libtmux.Window` iterating through session config dict.
 
-        Generator yielding :class:`Window` by iterating through
+        Generator yielding :class:`libtmux.Window` by iterating through
         ``sconf['windows']``.
 
         Applies ``window_options`` to window.
 
-        :param session: :class:`Session` from the config
-        :rtype: tuple(:class:`Window`, ``wconf``)
+        Parameters
+        ----------
+        session : :class:`libtmux.Session`
+            session to create windows in
 
+        Returns
+        -------
+        tuple of (:class:`libtmux.Window`, ``wconf``)
+            Newly created window, and the section from the tmuxp configuration
+            that was used to create the window.
         """
         for i, wconf in enumerate(self.sconf['windows'], start=1):
             if 'window_name' not in wconf:
@@ -249,7 +254,7 @@ class WorkspaceBuilder(object):
 
             if i == int(1) and w1:  # if first window, use window 1
                 w1.kill_window()
-            assert(isinstance(w, Window))
+            assert isinstance(w, Window)
             s.server._update_windows()
             if 'options' in wconf and isinstance(wconf['options'], dict):
                 for key, val in wconf['options'].items():
@@ -263,18 +268,25 @@ class WorkspaceBuilder(object):
             yield w, wconf
 
     def iter_create_panes(self, w, wconf):
-        """Return :class:`Pane` iterating through window config dict.
+        """
+        Return :class:`libtmux.Pane` iterating through window config dict.
 
         Run ``shell_command`` with ``$ tmux send-keys``.
 
-        :param w: window to create panes for
-        :type w: :class:`Window`
-        :param wconf: config section for window
-        :type wconf: :py:obj:`dict`
-        :rtype: tuple(:class:`Pane`, ``pconf``)
+        Parameters
+        ----------
+        w : :class:`libtmux.Window`
+            window to create panes for
+        wconf : dict
+            config section for window
 
+        Returns
+        -------
+        tuple of (:class:`libtmux.Pane`, ``pconf``)
+            Newly created pane, and the section from the tmuxp configuration
+            that was used to create the pane.
         """
-        assert(isinstance(w, Window))
+        assert isinstance(w, Window)
 
         pane_base_index = int(w.show_window_option('pane-base-index', g=True))
 
@@ -284,6 +296,7 @@ class WorkspaceBuilder(object):
             if pindex == int(pane_base_index):
                 p = w.attached_pane
             else:
+
                 def get_pane_start_directory():
 
                     if 'start_directory' in pconf:
@@ -292,13 +305,12 @@ class WorkspaceBuilder(object):
                         return wconf['start_directory']
                     else:
                         return None
+
                 p = w.split_window(
-                    attach=True,
-                    start_directory=get_pane_start_directory(),
-                    target=p.id
+                    attach=True, start_directory=get_pane_start_directory(), target=p.id
                 )
 
-            assert(isinstance(p, Pane))
+            assert isinstance(p, Pane)
             if 'layout' in wconf:
                 w.select_layout(wconf['layout'])
 
@@ -319,25 +331,38 @@ class WorkspaceBuilder(object):
 
             yield p, pconf
 
-    """
-    Applies window configurations relevant after window and pane creation.
-    """
     def config_after_window(self, w, wconf):
-        if (
-            'options_after' in wconf and
-            isinstance(wconf['options_after'], dict)
-        ):
+        """Actions to apply to window after window and pane finished.
+
+        When building a tmux session, sometimes its easier to postpone things
+        like setting options until after things are already structurally
+        prepared.
+
+        Parameters
+        ----------
+        w : :class:`libtmux.Window`
+            window to create panes for
+        wconf : dict
+            config section for window
+        """
+        if 'options_after' in wconf and isinstance(wconf['options_after'], dict):
             for key, val in wconf['options_after'].items():
                 w.set_window_option(key, val)
 
 
 def freeze(session):
-    """Freeze live tmux session and Return session config :py:obj:`dict`.
+    """
+    Freeze live tmux session and Return session config :py:obj:`dict`.
 
-    :param session: session object
-    :type session: :class:`Session`
-    :rtype: dict
+    Parameters
+    ----------
+    session : :class:`libtmux.Session`
+        session object
 
+    Returns
+    -------
+    dict
+        tmuxp compatible workspace config
     """
     sconf = {'session_name': session['session_name'], 'windows': []}
 
@@ -346,7 +371,7 @@ def freeze(session):
             'options': w.show_window_options(),
             'window_name': w.name,
             'layout': w.layout,
-            'panes': []
+            'panes': [],
         }
         if w.get('window_active', '0') == '1':
             wconf['focus'] = 'true'
@@ -354,10 +379,7 @@ def freeze(session):
         # If all panes have same path, set 'start_directory' instead
         # of using 'cd' shell commands.
         def pane_has_same_path(p):
-            return (
-                w.panes[0].current_path ==
-                p.current_path
-            )
+            return w.panes[0].current_path == p.current_path
 
         if all(pane_has_same_path(p) for p in w.panes):
             wconf['start_directory'] = w.panes[0].current_path
@@ -366,9 +388,7 @@ def freeze(session):
             pconf = {'shell_command': []}
 
             if 'start_directory' not in wconf:
-                pconf['shell_command'].append(
-                    'cd ' + p.current_path
-                )
+                pconf['shell_command'].append('cd ' + p.current_path)
 
             if p.get('pane_active', '0') == '1':
                 pconf['focus'] = 'true'
@@ -376,12 +396,8 @@ def freeze(session):
             current_cmd = p.current_command
 
             def filter_interpretters_and_shells():
-                return (
-                    current_cmd.startswith('-') or
-                    any(
-                        current_cmd.endswith(cmd)
-                        for cmd in ['python', 'ruby', 'node']
-                    )
+                return current_cmd.startswith('-') or any(
+                    current_cmd.endswith(cmd) for cmd in ['python', 'ruby', 'node']
                 )
 
             if filter_interpretters_and_shells():
